import java.util.concurrent.locks.*;


public class ConditionalSynchBank {
	private double[] accounts;
	private Lock myLock = new ReentrantLock();
	private Condition sufficientFunds = myLock.newCondition();
	
	/**
	 * 
	 * @param n
	 * @param initialbalance
	 */
	public ConditionalSynchBank(int n, double initialbalance)
	{
		accounts = new double[n];
		for(int i=0;i<n;i++)
		{
			accounts[i] = initialbalance;
		}
	}
	
	/**
	 * 
	 * @param from
	 * @param to
	 * @param amount
	 * @throws InterruptedException 
	 */
	public void transfer(int from, int to, double amount) throws InterruptedException
	{
		myLock.lock(); //locking access to the instructions in between.
		System.out.println(Thread.currentThread());
		while(accounts[from] < amount) sufficientFunds.await();
		accounts[from] -= amount;
		System.out.println("Transfer money from account "+from+" to account "+to+" amount:"+amount);
		accounts[to] += amount;
		System.out.println("Sum total: "+getTotalBalance());
		myLock.unlock(); //unlocking 
		sufficientFunds.notifyAll();
	}
	
	/**
	 * 
	 * @return
	 */
	public double getTotalBalance()
	{
		double sum = 0;
		for(int i=0;i< accounts.length;i++)
		{
			sum += accounts[i];
		}
		
		return sum;
	}
	
	public int getSize() { return accounts.length;}
}
